@@ -91,10 +91,10 @@ class BatchRetrieveClient(APIClient, Chunker):
 
         return data
 
-    def get_all_documents(
+    def _get_all_documents(
         self,
         dataset_id: str,
-        chunk_size: int = 1000,
+        chunksize: int = 1000,
         filters: List = [],
         sort: List = [],
         select_fields: List = [],
@@ -114,7 +114,7 @@ class BatchRetrieveClient(APIClient, Chunker):
         ----------
         dataset_id : string
             Unique name of dataset
-        chunk_size : list
+        chunksize: list
             Number of documents to retrieve per retrieval
         include_vector: bool
             Include vectors in the search results
@@ -135,7 +135,7 @@ class BatchRetrieveClient(APIClient, Chunker):
         number_of_documents = self.get_number_of_documents(
             dataset_id=dataset_id, filters=filters
         )
-        iterations_required = math.ceil(number_of_documents / chunk_size)
+        iterations_required = math.ceil(number_of_documents / chunksize)
 
         # While there is still data to fetch, fetch it at the latest cursor
 
@@ -146,7 +146,7 @@ class BatchRetrieveClient(APIClient, Chunker):
                 dataset_id,
                 filters=filters,
                 cursor=cursor,
-                page_size=chunk_size,
+                page_size=chunksize,
                 sort=sort,
                 select_fields=select_fields,
                 include_vector=include_vector,
